% Copyright © 2017 Universitat Jaume I (UJI) and Tampere University of Technology (TUT)
%
% Permission is hereby granted, free of charge, to any person obtaining a copy of
% this software and associated documentation files (the "Software”), to deal in
% the Software without restriction, including without limitation the rights to
% use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
% of the Software, and to permit persons to whom the Software is furnished to do
% so, subject to the following conditions:
%
% The above copyright notice and this permission notice shall be included in all
% copies or substantial portions of the Software.
%
% THE SOFTWARE IS PROVIDED "AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
% OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
% SOFTWARE.

function figure03(IS_OCTAVE)
% figure03 Draws and saves in './figures' folder Figure 3 presented in paper.
%
%
%   IS_OCTAVE: 0 -> matlab, 1 -> octave

	close all;
	

    if ~exist('./figures','dir')
        mkdir('./figures');
    end

	box1 = {};
	box1_accum = {};
	accum = {};
	databasename = 'libdb_52';
	macs         = 52;
	method       = 'remove_antennas_random';
	method 	     = [method, '_'];
	message      = 'Number of APs artificially disconnected';


	error    = [];
	distance = [];
	i=0;
	load(['results_' databasename '_k_001/normal/resume.mat']);
	error = [distance_accum(1:repetitions), accuracy_accum(1:repetitions), 0*ones(repetitions,1)];
	error_accum = [distances_accum(:), accuracies_accum(:), 0*ones(repetitions*testsamples,1)];
	i=i+1; box1{i} = distance_accum(1:repetitions); box2{i} = accuracy_accum(1:repetitions);
	box1_accum{i} = distances_accum(:); box2_accum{i} = accuracies_accum(:);
	for jj = 1:macs
	  load(['results_' databasename '_k_001/' method int2str(jj) '/resume.mat']);
	  error         = [error;[distance_accum(1:repetitions), accuracy_accum(1:repetitions), jj*ones(repetitions,1)]];
	  error_accum   = [error_accum;[distances_accum(:), accuracies_accum(:), jj*ones(repetitions*testsamples,1)]];
	  i=i+1; box1{i} = distance_accum(1:repetitions); box2{i} = accuracy_accum(1:repetitions);
	  box1_accum{i}  = distances_accum(:)'; box2_accum{i} = accuracies_accum(:)';
	  accum{i}  = distances_accum(:)'; box2_accum{i} = accuracies_accum(:)';
	end

	graphics_dir = [''];

	myxticklabels{1} = {'None'};
	for i = 1:macs
	  if strcmp(method,'remove_antenna')
		myxticklabels{i+1} = ['AP#' sprintf('%02d',i)];
	  else
		myxticklabels{i+1} = [sprintf('%02d',i) 'APs'];
	  end
	end

	%myplot = boxplot(cell2mat({box1{:}}')')
	%myplot = boxplot(cell2mat({box2{:}}')')
	myplot = boxplot(cell2mat({box1{:}}));
	%myplot = boxplot(cell2mat({box2_accum{:}}')')
	xidx = [1:8:(macs+1)];
	if IS_OCTAVE
		set(gca,'xtick',xidx ,'xticklabel',(myxticklabels(xidx)));
	else
	if IS_OCTAVE
		set(gca,'xtick',xidx ,'xticklabel',(myxticklabels(xidx)));
	else
		set(gca,'xtick',xidx ,'xticklabel',string(myxticklabels(xidx)));
	end
	end
	set(gca,'xlim',[0 (macs+2)]);
	set(gca,'ylim',[0 60]);
	%title(['Effect of disconnecting APs in the RSS distance']);
	xlabel(message,'fontsize', 14); my_ylim = ylim; my_xlim = xlim;
	ylabel('Average distance to the most similar ref. fingerprint (dBm)','fontsize', 14);
	%ylabel('Average positioning error (m)')
	daspect([ 1/(my_ylim(2)-my_ylim(1)) 1/(my_xlim(2)-my_xlim(1)) 1]);
	set(gca, 'fontsize', 14);

	print([ graphics_dir 'figures/figure03A'],'-dpdf');
	print([ graphics_dir 'figures/figure03A'],'-dpng');





	box1 = {};
	box1_accum = {};
	accum = {};
	databasename = 'libdb_52';
	macs         = 13;
	method       = 'remove_physicalantennas_random';

	if strcmp(method,'remove_antenna')
		%message = 'disconnecting a particular AP';
		message = 'AP artificially disconnected';
	elseif strcmp(method,'remove_physicalantenna')
		%message = 'disconnecting a particular AP';
		message = 'Physical device artificially disconnected';
	elseif strcmp(method,'remove_antennas_random')
		method = [method, '_'];
		%message = 'randomly disconnecting a given number of APs';
		message = 'Number of APs artificially disconnected';
	elseif strcmp(method,'remove_physicalantennas_random')
		method = [method, '_'];
		%message = 'randomly disconnecting a given number of APs';
		message = 'Number of physical devices artificially disconnected';
	elseif strcmp(method,'switch_antennas_random')
		method = [method, '_'];
		%message = 'randomly switching a given number of APs';
		message = 'Number of APs artificially switched';
	elseif strcmp(method,'switch_physicalantennas_random')
		method = [method, '_'];
		%message = 'randomly switching a given number of APs';
		message = 'Number of physical devices artificially switched';
	elseif strcmp(method,'switch_antennas')
		method = [method, '_'];
		%message = 'randomly switching a given number of APs';
		message = 'Case';
	elseif strcmp(method,'switch_physicalantennas')
		method = [method, '_'];
		%message = 'randomly switching a given number of APs';
		message = 'Case';
	end

	error    = [];
	distance = [];
	i=0;
	load(['results_' databasename '_k_001/normal/resume.mat']);
	error = [distance_accum(1:repetitions), accuracy_accum(1:repetitions), 0*ones(repetitions,1)];
	error_accum = [distances_accum(:), accuracies_accum(:), 0*ones(repetitions*testsamples,1)];
	i=i+1; box1{i} = distance_accum(1:repetitions); box2{i} = accuracy_accum(1:repetitions);
	box1_accum{i} = distances_accum(:); box2_accum{i} = accuracies_accum(:);
	for jj = 1:max(physicalaps)
	  load(['results_' databasename '_k_001/' method int2str(jj) '/resume.mat']);
	  error         = [error;[distance_accum(1:repetitions), accuracy_accum(1:repetitions), jj*ones(repetitions,1)]];
	  error_accum   = [error_accum;[distances_accum(:), accuracies_accum(:), jj*ones(repetitions*testsamples,1)]];
	  i=i+1; box1{i} = distance_accum(1:repetitions); box2{i} = accuracy_accum(1:repetitions);
	  box1_accum{i}  = distances_accum(:)'; box2_accum{i} = accuracies_accum(:)';
	  accum{i}  = distances_accum(:)'; box2_accum{i} = accuracies_accum(:)';
	end

	graphics_dir = [''];

	myxticklabels{1} = {'None'};
	for i = 1:macs
	  if strcmp(method,'remove_antenna')
		myxticklabels{i+1} = ['PAP#' sprintf('%02d',i)];
	  else
		myxticklabels{i+1} = [sprintf('%02d',i) 'PAPs'];
	  end
	end

	%myplot = boxplot(cell2mat({box1{:}}')')
	%myplot = boxplot(cell2mat({box2{:}}')')
	myplot = boxplot(cell2mat({box1{:}}));
	%myplot = boxplot(cell2mat({box2_accum{:}}')')
	xidx = [1:2:(max(physicalaps)+1)];
	if IS_OCTAVE
		set(gca,'xtick',xidx ,'xticklabel',(myxticklabels(xidx)));
	else
	if IS_OCTAVE
		set(gca,'xtick',xidx ,'xticklabel',(myxticklabels(xidx)));
	else
		set(gca,'xtick',xidx ,'xticklabel',string(myxticklabels(xidx)));
	end
	end
	set(gca,'xlim',[0 (max(physicalaps(:))+2)]);
	set(gca,'ylim',[0 60]);
	%title(['Effect of disconnecting APs in the RSS distance']);
	xlabel(message,'fontsize', 14);; my_ylim = ylim; my_xlim = xlim;
	ylabel('Average distance to the most similar ref. fingerprint (dBm)','fontsize', 14);
	%ylabel('Average positioning error (m)')
	daspect([ 1/(my_ylim(2)-my_ylim(1)) 1/(my_xlim(2)-my_xlim(1)) 1]);
	set(gca, 'fontsize', 14);

	print([ graphics_dir 'figures/figure03B'],'-dpdf');
	print([ graphics_dir 'figures/figure03B'],'-dpng');






	box1 = {};
	box1_accum = {};
	accum = {};
	databasename = 'libdb_13_2';
	macs    = 13;
	method       = 'remove_antennas_random';


	if strcmp(method,'remove_antenna')
		%message = 'disconnecting a particular AP';
		message = 'AP artificially disconnected';
	elseif strcmp(method,'move_antennas_random')
		method = [method, '_'];
		%message = 'randomly disconnecting a given number of APs';
		message = 'Number of APs displaced';
	elseif strcmp(method,'remove_antennas_random')
		method = [method, '_'];
		%message = 'randomly disconnecting a given number of APs';
		message = 'Number of APs artificially disconnected';
	elseif strcmp(method,'switch_antennas_random')
		method = [method, '_'];
		%message = 'randomly switching a given number of APs';
		message = 'Number of APs artificially switched';
	elseif strcmp(method,'switch_antennas')
		method = [method, '_'];
		%message = 'randomly switching a given number of APs';
		message = 'Case';
	end

	error    = [];
	distance = [];
	i=0;
	load(['results_' databasename '_k_001/normal/resume.mat'])
	error = [distance_accum(1:repetitions), accuracy_accum(1:repetitions), 0*ones(repetitions,1)];
	error_accum = [distances_accum(:), accuracies_accum(:), 0*ones(repetitions*testsamples,1)];
	i=i+1; box1{i} = distance_accum(1:repetitions); box2{i} = accuracy_accum(1:repetitions);
	box1_accum{i} = distances_accum(:); box2_accum{i} = accuracies_accum(:);
	for jj = 1:macs
	  load(['results_' databasename '_k_001/' method int2str(jj) '/resume.mat']);
	  error         = [error;[distance_accum(1:repetitions), accuracy_accum(1:repetitions), jj*ones(repetitions,1)]];
	  error_accum   = [error_accum;[distances_accum(:), accuracies_accum(:), jj*ones(repetitions*testsamples,1)]];
	  i=i+1; box1{i} = distance_accum(1:repetitions); box2{i} = accuracy_accum(1:repetitions);
	  box1_accum{i}  = distances_accum(:)'; box2_accum{i} = accuracies_accum(:)';
	  accum{i}  = distances_accum(:)'; box2_accum{i} = accuracies_accum(:)';
	end

	graphics_dir = [''];

	myxticklabels{1} = {'None'};
	for i = 1:macs
	  if strcmp(method,'remove_antenna')
		myxticklabels{i+1} = ['AP#' sprintf('%02d',i)];
	  else
		myxticklabels{i+1} = [sprintf('%02d',i) 'APs'];
	  end
	end

	%myplot = boxplot(cell2mat({box1{:}}')')
	%myplot = boxplot(cell2mat({box2{:}}')')
	myplot = boxplot(cell2mat({box1{:}}));
	%myplot = boxplot(cell2mat({box2_accum{:}}')')
	xidx = [1:2:(macs+1)];
	if IS_OCTAVE
		set(gca,'xtick',xidx ,'xticklabel',(myxticklabels(xidx)));
	else
	if IS_OCTAVE
		set(gca,'xtick',xidx ,'xticklabel',(myxticklabels(xidx)));
	else
		set(gca,'xtick',xidx ,'xticklabel',string(myxticklabels(xidx)));
	end
	end
	set(gca,'xlim',[0 (macs+2)]);set(gca,'ylim',[0 60]);
	%title(['Effect of disconnecting APs in the RSS distance']);
	xlabel(message,'fontsize', 14);; my_ylim = ylim; my_xlim = xlim;
	ylabel('Average distance to the most similar ref. fingerprint (dBm)','fontsize', 14);
	set(gca, 'fontsize', 14)

	%ylabel('Average positioning error (m)')
	daspect([ 1/(my_ylim(2)-my_ylim(1)) 1/(my_xlim(2)-my_xlim(1)) 1]);
	print([ graphics_dir 'figures/figure03C'],'-dpdf');
	print([ graphics_dir 'figures/figure03C'],'-dpng');






	box1 = {};
	box1_accum = {};
	accum = {};
	databasename = 'libdb_13_5';
	macs    = 13;
	method       = 'remove_antennas_random';


	if strcmp(method,'remove_antenna')
		%message = 'disconnecting a particular AP';
		message = 'AP artificially disconnected';
	elseif strcmp(method,'move_antennas_random')
		method = [method, '_'];
		%message = 'randomly disconnecting a given number of APs';
		message = 'Number of APs displaced';
	elseif strcmp(method,'remove_antennas_random')
		method = [method, '_'];
		%message = 'randomly disconnecting a given number of APs';
		message = 'Number of APs artificially disconnected';
	elseif strcmp(method,'switch_antennas_random')
		method = [method, '_'];
		%message = 'randomly switching a given number of APs';
		message = 'Number of APs artificially switched';
	elseif strcmp(method,'switch_antennas')
		method = [method, '_'];
		%message = 'randomly switching a given number of APs';
		message = 'Case';
	end

	error    = [];
	distance = [];
	i=0;
	load(['results_' databasename '_k_001/normal/resume.mat'])
	error = [distance_accum(1:repetitions), accuracy_accum(1:repetitions), 0*ones(repetitions,1)];
	error_accum = [distances_accum(:), accuracies_accum(:), 0*ones(repetitions*testsamples,1)];
	i=i+1; box1{i} = distance_accum(1:repetitions); box2{i} = accuracy_accum(1:repetitions);
	box1_accum{i} = distances_accum(:); box2_accum{i} = accuracies_accum(:);
	for jj = 1:macs
	  load(['results_' databasename '_k_001/' method int2str(jj) '/resume.mat']);
	  error         = [error;[distance_accum(1:repetitions), accuracy_accum(1:repetitions), jj*ones(repetitions,1)]];
	  error_accum   = [error_accum;[distances_accum(:), accuracies_accum(:), jj*ones(repetitions*testsamples,1)]];
	  i=i+1; box1{i} = distance_accum(1:repetitions); box2{i} = accuracy_accum(1:repetitions);
	  box1_accum{i}  = distances_accum(:)'; box2_accum{i} = accuracies_accum(:)';
	  accum{i}  = distances_accum(:)'; box2_accum{i} = accuracies_accum(:)';
	end

	graphics_dir = [''];

	myxticklabels{1} = {'None'};
	for i = 1:macs
	  if strcmp(method,'remove_antenna')
		myxticklabels{i+1} = ['AP#' sprintf('%02d',i)];
	  else
		myxticklabels{i+1} = [sprintf('%02d',i) 'APs'];
	  end
	end

	%myplot = boxplot(cell2mat({box1{:}}')')
	%myplot = boxplot(cell2mat({box2{:}}')')
	myplot = boxplot(cell2mat({box1{:}}));
	%myplot = boxplot(cell2mat({box2_accum{:}}')')
	xidx = [1:2:(macs+1)];
	if IS_OCTAVE
		set(gca,'xtick',xidx ,'xticklabel',(myxticklabels(xidx)));
	else
		set(gca,'xtick',xidx ,'xticklabel',string(myxticklabels(xidx)));
	end
	set(gca,'xlim',[0 (macs+2)]);set(gca,'ylim',[0 60]);
	%title(['Effect of disconnecting APs in the RSS distance']);
	xlabel(message,'fontsize', 14);; my_ylim = ylim; my_xlim = xlim;
	ylabel('Average distance to the most similar ref. fingerprint (dBm)','fontsize', 14);
	set(gca, 'fontsize', 14);

	%ylabel('Average positioning error (m)')
	daspect([ 1/(my_ylim(2)-my_ylim(1)) 1/(my_xlim(2)-my_xlim(1)) 1]);
	print([ graphics_dir 'figures/figure03D'],'-dpdf');
	print([ graphics_dir 'figures/figure03D'],'-dpng');


end


