% Copyright © 2017 Universitat Jaume I (UJI) and Tampere University of Technology (TUT)
%
% Permission is hereby granted, free of charge, to any person obtaining a copy of
% this software and associated documentation files (the "Software"), to deal in
% the Software without restriction, including without limitation the rights to
% use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
% of the Software, and to permit persons to whom the Software is furnished to do
% so, subject to the following conditions:
%
% The above copyright notice and this permission notice shall be included in all
% copies or substantial portions of the Software.
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
% OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
% SOFTWARE.

function figure04(IS_OCTAVE)
% figure04 Draws and saves in './figures' folder Figure 4 presented in paper.
%
%
%   IS_OCTAVE: 0 -> matlab, 1 -> octave

	close all;
	
    if ~exist('./figures','dir')
        mkdir('./figures');
    end

	rssi    = zeros(96,13);
	samples = zeros(96,13);

	minrssi = 0;
	maxrssi = -200;

	for i = 1:6:60

	  clear database; 

	  database = loadDB('libdb_13_2',1);

	  if min(database.trainingMacs(:)) < minrssi
		minrssi = min(database.trainingMacs(:));
	  end
	  if max(database.trainingMacs(database.trainingMacs<100)) > maxrssi
		maxrssi = max(database.trainingMacs(database.trainingMacs<100));
	  end

	  for j = 1:96

		values = database.trainingMacs((j-1)*6+[1:6],:);

		for k = 1:13
		  idx          = values(:,k) < 100;
		  rssi(j,k)    = rssi(j,k)    + sum(values(idx,k));
		  samples(j,k) = samples(j,k) + sum(idx,1);
		end


	  end

	end

	for j = 1:96

		labels(j,:) = database.trainingLabels((j-1)*6+1,:);

		if (floor((j-1)/24) == 1) || (floor((j-1)/24) == 3)
		   labels(j,1) = labels(j,1)+0.25;
		else
		   labels(j,1) = labels(j,1)-0.25 ;
		end


	end




	for k = 6

	  idx      = samples(:,k)>0;
	  samples1 = samples(idx,k);
	  rssi1    = rssi(idx,k)./samples1;
	  labels1  = labels(idx,:);

	  idx3F     = labels1(:,4) == 3;
	  samples3F = samples1(idx3F);
	  rssi3F    = rssi1(idx3F);
	  labels3F  = labels1(idx3F,:);

	  idx5F     = labels1(:,4) == 5;
	  samples5F = samples1(idx5F);
	  rssi5F    = rssi1(idx5F);
	  labels5F  = labels1(idx5F,:);


	  minrssi    = -30;%min(rssi(:));
	  maxrssi    = -110;%max(rssi(:));
	  minsamples = 0;
	  maxsamples = 60;

	  rssi3F    = [rssi3F;minrssi;maxrssi];
	  rssi5F    = [rssi5F;minrssi;maxrssi];
	  samples3F = [samples3F;minsamples;maxsamples];
	  samples5F = [samples5F;minsamples;maxsamples];
	  labels3F  = [labels3F;(zeros(2,5)-100)];
	  labels5F  = [labels5F;(zeros(2, 5)-100)];

	  close all;
	  myfig = figure;
	  s1 = subplot(2,1,2);
	  if sum(idx3F,1)>0
		scatter(labels(:,2), labels(:,1), 10, 'x'); hold on;
		scatter(labels3F(:,2), labels3F(:,1), 60+0*samples3F, samples3F/60*100, 'filled');
	  end;
	  colorbar;
	  colormap jet;
	  title('3rd Floor (2.4 GHz)');
	  set(gca,'xtick',[16:2:30],'xticklabel',[16:2:30]);
	  set(gca,'xlim',[16 30]);
	  set(gca,'ytick',[3:2:14],'yticklabel',[3:2:14]);
	  set(gca,'ylim',[3 14]);
	  daspect([1 1 1]);
	  s2 = subplot(2,1,1);
	  if sum(idx5F,1)>0
		  scatter( labels(:,2), labels(:,1), 10, 'x'); hold on;
		  scatter( labels5F(:,2), labels5F(:,1), 60+0*samples5F, samples5F/60*100, 'filled');
	  end;
	  colorbar;
	  colormap jet;
	  title('5th Floor (2.4 GHz)');
	  set(gca,'xtick',[16:2:30],'xticklabel',[16:2:30]);
	  set(gca,'xlim',[16 30]);
	  set(gca,'ytick',[3:2:14],'yticklabel',[3:2:14]);
	  set(gca,'ylim',[3 14]);
	  daspect([1 1 1]);
	  print(myfig,['figures/figure04A'],'-dpdf');
	  print(myfig,['figures/figure04A'],'-dpng');
	  close all

	end

	clear;
	clc;
	rssi    = zeros(96,13);
	samples = zeros(96,13);

	minrssi = 0;
	maxrssi = -200;

	for i = 1:6:60

	  clear database;

	  database = loadDB('libdb_13_5',1);

	  if min(database.trainingMacs(:)) < minrssi
		minrssi = min(database.trainingMacs(:));
	  end
	  if max(database.trainingMacs(database.trainingMacs<100)) > maxrssi
		maxrssi = max(database.trainingMacs(database.trainingMacs<100));
	  end

	  for j = 1:96

		values = database.trainingMacs((j-1)*6+[1:6],:);

		for k = 1:13
		  idx          = values(:,k) < 100;
		  rssi(j,k)    = rssi(j,k)    + sum(values(idx,k));
		  samples(j,k) = samples(j,k) + sum(idx,1);
		end


	  end

	end

	for j = 1:96

		labels(j,:) = database.trainingLabels((j-1)*6+1,:);

		if (floor((j-1)/24) == 1) || (floor((j-1)/24) == 3)
		   labels(j,1) = labels(j,1)+0.25;
		else
		   labels(j,1) = labels(j,1)-0.25;
		end


	end




	for k = 6

	  idx      = samples(:,k)>0;
	  samples1 = samples(idx,k);
	  rssi1    = rssi(idx,k)./samples1;
	  labels1  = labels(idx,:);

	  idx3F     = labels1(:,4) == 3;
	  samples3F = samples1(idx3F);
	  rssi3F    = rssi1(idx3F);
	  labels3F  = labels1(idx3F,:);

	  idx5F     = labels1(:,4) == 5;
	  samples5F = samples1(idx5F);
	  rssi5F    = rssi1(idx5F);
	  labels5F  = labels1(idx5F,:);


	  minrssi    = -30;%min(rssi(:));
	  maxrssi    = -110;%max(rssi(:));
	  minsamples = 0;
	  maxsamples = 60;

	  rssi3F    = [rssi3F;minrssi;maxrssi];
	  rssi5F    = [rssi5F;minrssi;maxrssi];
	  samples3F = [samples3F;minsamples;maxsamples];
	  samples5F = [samples5F;minsamples;maxsamples];
	  labels3F  = [labels3F;(zeros(2,5)-100)];
	  labels5F  = [labels5F;(zeros(2, 5)-100)];

	  close all;
	  myfig = figure;
	  s1 = subplot(2,1,2);
	  if sum(idx3F,1)>0
		scatter(labels(:,2), labels(:,1), 10, 'x'); hold on;
		scatter(labels3F(:,2), labels3F(:,1), 60+0*samples3F, samples3F/60*100, 'filled');
	  end;
	  colorbar;
	  colormap jet;
	  title('3rd Floor (5.2 GHz)');
	  set(gca,'xtick',[16:2:30],'xticklabel',[16:2:30]);
	  set(gca,'xlim',[16 30]);
	  set(gca,'ytick',[3:2:14],'yticklabel',[3:2:14]);
	  set(gca,'ylim',[3 14]);
	  daspect([1 1 1]);
	  s2 = subplot(2,1,1);
	  if sum(idx5F,1)>0
		  scatter( labels(:,2), labels(:,1), 10, 'x'); hold on;
		  scatter( labels5F(:,2), labels5F(:,1), 60+0*samples5F, samples5F/60*100, 'filled');
	  end;
	  colorbar;
	  colormap jet;
	  title('5th Floor (5.2 GHz)');
	  set(gca,'xtick',[16:2:30],'xticklabel',[16:2:30]);
	  set(gca,'xlim',[16 30]);
	  set(gca,'ytick',[3:2:14],'yticklabel',[3:2:14]);
	  set(gca,'ylim',[3 14]);
	  daspect([1 1 1]);
	  print(myfig,['figures/figure04B'],'-dpdf');
	  print(myfig,['figures/figure04B'],'-dpng');
	  close all

	end
  
 end
  
  