% Copyright © 2017 Universitat Jaume I (UJI) and Tampere University of Technology (TUT)
%
% Permission is hereby granted, free of charge, to any person obtaining a copy of
% this software and associated documentation files (the "Software"), to deal in
% the Software without restriction, including without limitation the rights to
% use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
% of the Software, and to permit persons to whom the Software is furnished to do
% so, subject to the following conditions:
%
% The above copyright notice and this permission notice shall be included in all
% copies or substantial portions of the Software.
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
% OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
% SOFTWARE.

function figure07(IS_OCTAVE)
% figure07 Draws and saves in './figures' folder Figure 7 presented in paper.
%
%
%   IS_OCTAVE: 0 -> matlab, 1 -> octave

	close all;

    if ~exist('./figures','dir')
        mkdir('./figures');
    end

	databasename = 'libdb_13_2';
	method       = 'remove_antenna_with_spatial_analysis';



	 %pkg load statistics

	mycolormap = [0,1,0;1,1,1;0,0,1;1,0,0];

	message = 'AP artificially disconnected';


	error    = [];
	distance = [];
	i=0;
	% load results from file
	load(['results_' databasename '_k_003/normal_with_spatial_analysis/resume.mat']);
	error = [distance_accum(1:repetitions), accuracy_accum(1:repetitions), 0*ones(repetitions,1)];
	%error_accum = [distances_accum(:), accuracies_accum(:), 0*ones(repetitions*testsamples,1)];
	i=i+1;
	box1{i} = distance_accum(1:repetitions)'; box2{i} = accuracy_accum(1:repetitions)';
	box1_accum{i} = distances_accum(:)'; box2_accum{i} = accuracies_accum(:)';
	N{i} = NperArea_accum;
	for jj = 1:max(physicalaps)
	  load(['results_' databasename '_k_003/' method int2str(jj) '/resume.mat']);
	  error         = [error;[distance_accum(1:repetitions), accuracy_accum(1:repetitions), jj*ones(repetitions,1)]];
	%  error_accum   = [error_accum;[distances_accum(:), accuracies_accum(:), jj*ones(repetitions*testsamples,1)]];
	  i=i+1; box1{i} = distance_accum(1:repetitions); box2{i} = accuracy_accum(1:repetitions);
	  box1_accum{i}  = distances_accum(:)'; box2_accum{i} = accuracies_accum(:)';
	  accum{i}  = distances_accum(:)'; box2_accum{i} = accuracies_accum(:)';
	  N{i} = NperArea_accum;
	end

	graphics_dir = ['./figures/'];
	if ~exist(graphics_dir)
		mkdir(graphics_dir);
	end

	repetition = length(NperArea_accum); % -> chose the last run to plot
	numFl = size(NperArea_accum{repetition}, 1);
	for jj = 1:max(physicalaps)+1
		for iFl = 1:numFl
			[gX,gY] = gradient(N{jj}{repetition}{iFl});
			M{jj,iFl} = ~logical(gX+gY);
			%fprintf('no. of cells: %i\n', sum(sum(M)));
			% get coordinates for axis !! not working with imagesc !!
			[XX,YY] = meshgrid(xEdges,yEdges);
			% adapt choice of dimension above
			XX = XX(2:end,2:end);
			YY = YY(2:end,2:end);
			if numFl > 1
				subplot(2,1,iFl);
			end
			if isempty(M{jj,iFl})
				M = zeros(size(XX));
			end
			pcolor(YY,XX,double(M{1,iFl} - M{jj,iFl}));
			ylabel('x (m)');
			xlabel('y (m)');
			if iFl == 1
				title('5th Floor');
			else
				title('3rd Floor');
			end
			axis image;
			colormap(mycolormap);
		end
		print([ graphics_dir 'figure07_AP',num2str(jj-1),'.pdf'],'-dpdf');
		print([ graphics_dir 'figure07_AP',num2str(jj-1),'.png'],'-dpng');
	end
	close all;
end


