% Copyright © 2017 Universitat Jaume I (UJI) and Tampere University of Technology (TUT)
%
% Permission is hereby granted, free of charge, to any person obtaining a copy of
% this software and associated documentation files (the "Software"), to deal in
% the Software without restriction, including without limitation the rights to
% use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
% of the Software, and to permit persons to whom the Software is furnished to do
% so, subject to the following conditions:
%
% The above copyright notice and this permission notice shall be included in all
% copies or substantial portions of the Software.
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
% OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
% SOFTWARE.

function figure11(IS_OCTAVE)
% figure11 Draws and saves in './figures' folder Figure 11 presented in paper.
%
%
%   IS_OCTAVE: 0 -> matlab, 1 -> octave

	close all;

	if IS_OCTAVE
		pkg load statistics;
	end

	antennas = [
		-23.626,-18.596,1;
		8.538,-9.298,3;
		-10.702,-18.596,4;
		-1.93,-2.749,6;
		8.596,-14.62,8;
		7.135,6.023,9;
		21.17,-2.69,15;
		13.333,-2.69,16;
		32.398,-2.69,17;
		32.573,13.86,25
	];

	mycolormap = [0,1,0;1,1,1;0,0,1;1,0,0];
	mycolormapred = [1,1,1;0,0,1];


	k = 5; % k for NN
	faultType = 'remove_antenna'; % remove_antenna % remove_antennas_random


	% parameters for 2D histogram
	xDim = [-27,34]; %[min(trn.coords(:,1)), max(trn.coords(:,1))];
	yDim = [-23,14]; %[min(trn.coords(:,2)), max(trn.coords(:,2))];
	res = 1; % resolution for cells
	X = (floor(xDim(1)): res :ceil(xDim(2)) + 1)';
	Y = (floor(yDim(1)): res :ceil(yDim(2)) + 1)';
	totNoCells = numel(Y) * numel(X);

	% preallocate memory
	numAp = 11;
	posEst = cell(1, numAp+1);
	leastKDist = cell(1, numAp+1);
	noCellsDiff = zeros(1, numAp+1);
	N = cell(1,numAp+1);
	M = cell(1,numAp+1);

	results_dir = ['./results_mandb_reduced_spatial_analysis_k_' sprintf('%03d',k') '/', faultType '/'];
	 
	for iR = 1:numAp+1 % iR == 1 equals no fault case
		
		load([results_dir  '/results_spatialanalysis_' sprintf('%03d',iR-1) '.mat'],'dist','sd','i','leastKDist','leastKInd','posEst','ip');

		% 2D historgram
		N{iR} = myhistcounts2(posEst{iR}(:,1),posEst{iR}(:,2),X,Y).';
		% gradient of histogram
		[gX,gY] = gradient(N{iR});
		% convert to logical values
		M{iR} = logical(gX+gY);
		noCellsDiff(iR) = numel(find(M{1}-M{iR}));
		% get coordinates back after storing counts in N:
		[XX,YY] = meshgrid(X,Y);
		% adapt to choice of dimension above
		XX = XX(2:end,2:end);
		YY = YY(2:end,2:end);
		if isempty(M{iR})
			M{iR} = zeros(XX);
		end

		close all;
		h=figure(iR); hold on;
		pcolor(XX,YY,double(M{1}-0.5*M{iR}));
		shading flat;
		daspect([1 1 1]); %colorbar;
		scatter((antennas(:,1)),(antennas(:,2)),'filled','MarkerEdgeColor',[0 .5 .5],...
			 'MarkerFaceColor',[0 .7 .7],...
			 'LineWidth',1.5);
		if  noCellsDiff(iR) == 0
			colormap(mycolormapred);
		else
			colormap(mycolormap);
		end
		set(gca,'xlim',[xDim(1)+1 xDim(2)+1]);
		set(gca,'ylim',[yDim(1)+1 yDim(2)+1]);
		set(gca,'xtick',-26:4:600 ,'xticklabel',-26:4:34);
		set(gca,'ytick',-22:4:600 ,'yticklabel',-22:4:14);
			if iR == 1 % nominal case
				print(h,[ './figures/figure11_nominal.pdf'],'-dpdf');
				print(h,[ './figures/figure11_nominal.png'],'-dpng');
			else
				print(h,[ './figures/figure11_AP' sprintf('%02d',iR-1) '.pdf'],'-dpdf');
				print(h,[ './figures/figure11_AP' sprintf('%02d',iR-1) '.png'],'-dpng');                
			end
		close all;

    
	end % end remove AP	
end
