% Copyright © 2017 Universitat Jaume I (UJI) and Tampere University of Technology (TUT)
%
% Permission is hereby granted, free of charge, to any person obtaining a copy of
% this software and associated documentation files (the "Software"), to deal in
% the Software without restriction, including without limitation the rights to
% use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
% of the Software, and to permit persons to whom the Software is furnished to do
% so, subject to the following conditions:
%
% The above copyright notice and this permission notice shall be included in all
% copies or substantial portions of the Software.
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
% OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
% SOFTWARE.

function generateDB_libdb_52(IS_OCTAVE)
% generateDB_libdb_52 Creates '\databases_singlefile\libdb_52.mat' with
% 60 new databases from the 10 months and 6 samples per reference point in 
% the Library Database, considering only 52 well identified antennas.
%
%   The colums are sorted so that columns c,c+13,c+26,c+39 represent APs
%   emitted by the same device (1<=i<=13), with c and c+13 networks emitted
%   at 2.4 GHz and c+26 and c+39 networks emitted at 5 GHz.
%
%   IS_OCTAVE: 0 -> matlab, 1 -> octave

    if ~exist('databases_singlefile','dir')
        mkdir('databases_singlefile');
    end

    basefolder = '../databases/libdb_files';

    apdata = getAPLibDataInfo();

    for month = 1:10

        folder = [basefolder '/' sprintf('%02d',month) '/'];

        train_labels = load([folder 'trn01crd.csv']);
        test1_labels = load([folder 'tst01crd.csv']);
        test2_labels = load([folder 'tst02crd.csv']);
        test3_labels = load([folder 'tst03crd.csv']);
        test4_labels = load([folder 'tst04crd.csv']);
        test5_labels = load([folder 'tst05crd.csv']);	

        train_rss0 = load([folder 'trn01rss.csv']);
        test1_rss = load([folder 'tst01rss.csv']);
        test2_rss = load([folder 'tst02rss.csv']);
        test3_rss = load([folder 'tst03rss.csv']);
        test4_rss = load([folder 'tst04rss.csv']);
        test5_rss = load([folder 'tst05rss.csv']);	

        test_labels = [test1_labels;test2_labels;test3_labels;test4_labels;test5_labels;];
        test_rss0 = [test1_rss;test2_rss;test3_rss;test4_rss;test5_rss;];

        train_rss = zeros(size(train_rss0,1),26);
        test_rss  = zeros(size(test_rss0,1),26);
        
        
        for antenna = 1:13
            idx   =     apdata( ((apdata(:,4)==antenna).*(apdata(:,3)<3000))>0 ,1); % 2.4 GHz and specific device
            idx2  =     apdata( ((apdata(:,4)==antenna).*(apdata(:,3)>3000))>0 ,1); % 5 GHz and specific device

            train_rss(:,antenna)    = train_rss0(:,idx(1));
            train_rss(:,antenna+13) = train_rss0(:,idx(2));    
            train_rss(:,antenna+26) = train_rss0(:,idx2(1));
            train_rss(:,antenna+39) = train_rss0(:,idx2(2));

            test_rss(:,antenna)    = test_rss0(:,idx(1));
            test_rss(:,antenna+13) = test_rss0(:,idx(2));    
            test_rss(:,antenna+26) = test_rss0(:,idx2(1));
            test_rss(:,antenna+39) = test_rss0(:,idx2(2));
        end

        for i = 1:6
            database.trainingMacs   = train_rss;
            database.trainingLabels = train_labels;
            database.testMacs       = test_rss(i:6:end,:);
            database.testLabels     = test_labels(i:6:end,:);

            database.trainingLabels(:,4) = database.trainingLabels(:,3);
            database.testLabels(:,4)     = database.testLabels(:,3);

            database.trainingLabels(:,5) = 0;
            database.testLabels(:,5)     = 0;

            database.trainingLabels(:,3) = 3.7*3+(database.trainingLabels(:,4)-3)*3.7/2;
            database.testLabels(:,3)     = 3.7*3+(database.testLabels(:,4)-3)*3.7/2;

            databases{(month-1)*6+i} = database;
        end
    end

    if IS_OCTAVE
        save(['databases_singlefile/libdb_52.mat'],'databases','-mat7-binary');
    else
        save(['databases_singlefile/libdb_52.mat'],'databases');
    end
	
	disp('Generation done for .\databases_singlefile\libdb_52.mat');
end
