% Copyright © 2017 Universitat Jaume I (UJI) and Tampere University of Technology (TUT)
%
% Permission is hereby granted, free of charge, to any person obtaining a copy of
% this software and associated documentation files (the "Software"), to deal in
% the Software without restriction, including without limitation the rights to
% use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
% of the Software, and to permit persons to whom the Software is furnished to do
% so, subject to the following conditions:
%
% The above copyright notice and this permission notice shall be included in all
% copies or substantial portions of the Software.
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
% OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
% SOFTWARE.

function [ database ] = loadDB_mandb_reduced( instance )

	if ~(exist('databases_singlefile/mandb_reduced.mat','file'))

		trainingFile = '../databases/mandb_files/1.5meters.offline.trace.txt';
		testFile     = '../databases/mandb_files/1.5meters.online.trace.txt';
		  

		Macs = loadDB_obtainMacs(trainingFile,testFile);

		printf('Macs loaded\n');

		[database.trainingMacs,database.trainingLabels,database.trainingOthers] = loadDB_generateMatrixes(trainingFile,Macs);
		[database.testMacs,database.testLabels,database.testOthers]             = loadDB_generateMatrixes(testFile,Macs);


		printf('Preproceses Database loaded\n');


		idx = 1;
		for i = 2:size(database.trainingMacs,1)

		  if    ((abs(database.trainingLabels(i,1)-database.trainingLabels(i-1,1))>0.001)||...
				 (abs(database.trainingLabels(i,2)-database.trainingLabels(i-1,2))>0.001)||...
				 (abs(database.trainingLabels(i,3)-database.trainingLabels(i-1,3))>0.001)||...
				 (abs(database.trainingLabels(i,4)-database.trainingLabels(i-1,4))>2))
			  idx = [idx, i];
		  end
		  i
		end

		idx2 = 1;
		for i = 2:size(database.testMacs,1)
		  if    ((abs(database.testLabels(i,1)-database.testLabels(i-1,1))>0.001)||...
				 (abs(database.testLabels(i,2)-database.testLabels(i-1,2))>0.001)||...
				 (abs(database.testLabels(i,3)-database.testLabels(i-1,3))>0.001)||...
				 (abs(database.testLabels(i,4)-database.testLabels(i-1,4))>2))
			idx2 = [idx2, i];
		  end
		  i
        end

		idx = idx(1:(end-1));
		idx2 = idx2(1:(end-1));


		database0 = database;
		database0.trainingLabels(:,5)=0;
		database0.testLabels(:,5)=0;
		clear database;


		idx_training = csvread(['mannheim2_idx/mannheim2_trainidxs' sprintf('%03d',instance) '.csv']);
		idx_test     = csvread(['mannheim2_idx/mannheim2_testidxs' sprintf('%03d',instance) '.csv']);


		validMacs = [1,3,4,6,8,9,13,15,16,17,25];
		
		database.trainingMacs   = database0.trainingMacs(idx_training,validMacs);
		database.testMacs       = database0.testMacs(idx_test,validMacs);
		database.trainingLabels = database0.trainingLabels(idx_training,:);
		database.testLabels     = database0.testLabels(idx_test,:);
		
    else
        load('databases_singlefile/mandb_reduced.mat','databases');
        database = databases{instance};
    end


end