% Copyright © 2017 Universitat Jaume I (UJI) and Tampere University of Technology (TUT)
%
% Permission is hereby granted, free of charge, to any person obtaining a copy of
% this software and associated documentation files (the "Software"), to deal in
% the Software without restriction, including without limitation the rights to
% use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
% of the Software, and to permit persons to whom the Software is furnished to do
% so, subject to the following conditions:
%
% The above copyright notice and this permission notice shall be included in all
% copies or substantial portions of the Software.
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
% OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
% SOFTWARE.

function  run_remove_antenna_mandb_reduced_spatial_analysis(k,IS_OCTAVE)

if IS_OCTAVE
    pkg load statistics
end

antennas = [-23.626,-18.596,1;
    8.538,-9.298,3;
    -10.702,-18.596,4;
    -1.93,-2.749,6;
    8.596,-14.62,8;
    7.135,6.023,9;
    21.17,-2.69,15;
    13.333,-2.69,16;
    32.398,-2.69,17;
    32.573,13.86,25];

mycolormap = [0,1,0;1,1,1;0,0,1;1,0,0];
mycolormapred = [1,1,1;0,0,1];

PLOT = 1;
WRITE= 1;

db_dir = '../databases/mandb_files/';

trainingFile = '1.5meters.offline.trace.txt';
testFile     = '1.5meters.online.trace.txt';

faultType = 'remove_antenna'; % remove_antenna % remove_antennas_random

if WRITE
    save_dir = ['./results_mandb_reduced_spatial_analysis_k_' sprintf('%03d',k') '/', faultType '/'];
    if ~exist(save_dir, 'dir')
        mkdir(save_dir);
    end
end

if ~exist('./figures','dir')
    mkdir('./figures')
end

tmpDir = ['tmp'];
if ~exist(tmpDir,'dir')
    mkdir(tmpDir)
end

tmpDir = ['tmp/' faultType 'final'];
if ~exist(tmpDir,'dir')
    mkdir(tmpDir)
end


if exist('tmp/macs_temp.mat')
    load('tmp/macs_temp.mat')
else
  macs = loadDB_mandb_obtainMacs([db_dir, trainingFile],[db_dir, testFile]);
  if (IS_OCTAVE)
      save('tmp/macs_temp.mat','macs','-binary');
  else
      save('tmp/macs_temp.mat','macs');      
  end
end


if exist('tmp/database_temp.mat')
  load('tmp/database_temp.mat')
else
  [trn.rss, trn.coords, trn.others] = loadDB_mandb_generateMatrixes([db_dir, trainingFile], macs);
  [tst.rss, tst.coords, tst.others] = loadDB_mandb_generateMatrixes([db_dir, testFile], macs);
    if (IS_OCTAVE)
      save('tmp/database_temp.mat','trn', 'tst','-binary')
  else
      save('tmp/database_temp.mat','trn', 'tst')
  end
    
end

validMacs = [1,3,4,6,8,9,13,15,16,17,25];
trn.rss = trn.rss(:,validMacs);
tst.rss = tst.rss(:,validMacs);

% pick single value from training data
[idx, ~] = find(diff(trn.coords(:,1:2)));
idxUnique = unique(idx);
trn.rss = trn.rss(idxUnique,:); % pick last RSS from set of 110 RSS values
trn.coords = trn.coords(idxUnique,:);
trn.others = trn.others(idxUnique,:);

rssMinTot = min(min(trn.rss)); % lowest RSS value, e.g. -98 dBm

trn.rss(trn.rss==100) = rssMinTot-1;
tst.rss(tst.rss==100) = rssMinTot-1;

% parameters for 2D histogram
xDim = [-27,34]; %[min(trn.coords(:,1)), max(trn.coords(:,1))];
yDim = [-23,14]; %[min(trn.coords(:,2)), max(trn.coords(:,2))];
res = 1; % resolution for cells
X = (floor(xDim(1)): res :ceil(xDim(2)) + 1)';
Y = (floor(yDim(1)): res :ceil(yDim(2)) + 1)';
totNoCells = numel(Y) * numel(X);
% copy without errors
tstOrig.rss = tst.rss;

% preallocate memory
numAp = size(trn.rss,2);
posEst = cell(1, numAp+1);
leastKDist = cell(1, numAp+1);
noCellsDiff = zeros(1, numAp+1);
N = cell(1,numAp+1);
M = cell(1,numAp+1);

for iR = 1:numAp+1 % iR == 1 equals no fault case
    
    % remove certain AP
    if iR > 1
        idxMod = iR-1;
        tst.rss = tstOrig.rss;
        tst.rss (:, idxMod) = rssMinTot-1;
    end
    
    len = length(tst.rss);
    dist = nan(1, size(trn.rss,1));
    leastKDist{iR} = nan(len, 1);
    posEst{iR} = nan(len, 2);

        for ip = 1:len
            dist(ip,:) = sqrt(sum((tst.rss(ip,:) - trn.rss).^2, 2)).';
            [sd,i] = sort(dist(ip,:),2);
            leastKDist{iR}(ip,:) = min(sd(:,1:k));
            leastKInd = i(:,1:k);
            % average over k candidates
            posEst{iR}(ip,:) = mean(trn.coords(leastKInd,1:2), 1);
        end
        if IS_OCTAVE
            save([save_dir '/results_spatialanalysis_' sprintf('%03d',iR-1) '.mat'],'dist','sd','i','leastKDist','leastKInd','posEst','ip','-binary');
        else
            save([save_dir  '/results_spatialanalysis_' sprintf('%03d',iR-1) '.mat'],'dist','sd','i','leastKDist','leastKInd','posEst','ip');
		end
    
    % 2D historgram
    N{iR} = myhistcounts2(posEst{iR}(:,1),posEst{iR}(:,2),X,Y).';
    % gradient of histogram
    [gX,gY] = gradient(N{iR});
    % convert to logical values
    M{iR} = logical(gX+gY);
    noCellsDiff(iR) = numel(find(M{1}-M{iR}));
    % get coordinates back after storing counts in N:
    [XX,YY] = meshgrid(X,Y);
    % adapt to choice of dimension above
    XX = XX(2:end,2:end);
    YY = YY(2:end,2:end);
    if isempty(M{iR})
        M{iR} = zeros(XX);
    end

%     if PLOT
%         h=figure(iR); hold on;
%         pcolor(XX,YY,double(M{1}-0.5*M{iR}));
%         shading flat;
%         daspect([1 1 1]); %colorbar;
%         scatter((antennas(:,1)),(antennas(:,2)),'filled','MarkerEdgeColor',[0 .5 .5],...
%              'MarkerFaceColor',[0 .7 .7],...
%              'LineWidth',1.5);
%         if  noCellsDiff(iR) == 0
%             colormap(mycolormapred);
%         else
%             colormap(mycolormap);
%         end
%         set(gca,'xlim',[xDim(1)+1 xDim(2)+1]);
%         set(gca,'ylim',[yDim(1)+1 yDim(2)+1]);
%         set(gca,'xtick',-26:4:600 ,'xticklabel',-26:4:34);
%         set(gca,'ytick',-22:4:600 ,'yticklabel',-22:4:14);
%         if WRITE
%             if iR == 1 % nominal case
%                 print(h,[ './figures/figure11_nominal.pdf'],'-dpdf');
%                 print(h,[ './figures/figure11_nominal.png'],'-dpng');
%             else
%                 print(h,[ './figures/figure11_AP' sprintf('%02d',iR-1) '.pdf'],'-dpdf');
%                 print(h,[ './figures/figure11_AP' sprintf('%02d',iR-1) '.png'],'-dpng');                
%             end
%         end
%         close all;
%     end
    
end % end remove AP
