% Copyright © 2017 Universitat Jaume I (UJI) and Tampere University of Technology (TUT)
%
% Permission is hereby granted, free of charge, to any person obtaining a copy of
% this software and associated documentation files (the "Software"), to deal in
% the Software without restriction, including without limitation the rights to
% use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
% of the Software, and to permit persons to whom the Software is furnished to do
% so, subject to the following conditions:
%
% The above copyright notice and this permission notice shall be included in all
% copies or substantial portions of the Software.
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
% OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
% SOFTWARE.

function run_remove_antenna_with_spatial_analysis( databasename ,  removedantenna, k, IS_OCTAVE)

	more off;

	% generate output folder
	prefix_results = ['remove_antenna_with_spatial_analysis' int2str(removedantenna)] ;
	dir_output     = ['results_'  databasename '_k_' sprintf('%03d',k) '/' prefix_results ];
	if ~exist(dir_output)
		mkdir(dir_output);
	end

 	% Set number of repetitions
	repetitions = setup_getRepetitions(databasename);

	% load first subset to get some basic database features
	database    = loadDB(databasename, 1);
	testsamples = size(database.testMacs,1);

	% generate output data structures
	accuracy_accum   = zeros(repetitions,1);
	distance_accum   = zeros(repetitions,1);
%	accuracies_accum = zeros(repetitions,testsamples);
%	distances_accum  = zeros(repetitions,testsamples);

	% modify macs according to macconfiguration and dataset
	% this can be used to filter noisy APs or APs in other
	% buildings/floors/areas
	physicalaps = setup_getPhysicalAP(databasename,database); 

	for i = 1:repetitions

		database0 = loadDB(databasename, i);

        % set-up for 2D histogram
        xDim = [min(database0.trainingLabels(:,1)), max(database0.trainingLabels(:,1))];
        yDim = [min(database0.trainingLabels(:,2)), max(database0.trainingLabels(:,2))];
        res = 0.5; % size of 2D bins in m
        xEdges = (floor(xDim(1)): res :ceil(xDim(2))+1)';
        yEdges = (floor(yDim(1)): res :ceil(yDim(2))+1)';

        numFl = numel(unique(database0.trainingLabels(:,4))); % 4th column height
        NperArea = cell(numFl,1);
        resPerFloor = cell(numFl,1);
        floorHgt = unique(database0.trainingLabels(:,4));
		clear database;

        % filter per floor
        for iFl = 1:numFl
            idxFl{iFl} = find(floorHgt(iFl) == database0.trainingLabels(:,4));

            database.trainingMacs   = database0.trainingMacs(idxFl{iFl},:);
            database.trainingLabels = database0.trainingLabels(idxFl{iFl},:);
            database.testMacs       = database0.testMacs(:,:);
            database.testLabels     = database0.testLabels;

            database.testMacs(:,removedantenna) = +100;

            % Default k-NN with k=1, positive data representation
            % euclidean distance to compare fingerprints, and default
            % non-detected RSS value = 100
            resPerFloor{iFl} = knn_baseline(database,k,100,[]);

            % compute estimate from knn posEst for each test point
            NperArea{iFl} = myhistcounts2(resPerFloor{iFl}.prediction(:,1),resPerFloor{iFl}.prediction(:,2),xEdges,yEdges).';
        end

        if numFl == 2 % only for 2 floors so far
            results.candidates = vertcat(resPerFloor{1}.candidates, resPerFloor{2}.candidates);
            results.distances = vertcat(resPerFloor{1}.distances, resPerFloor{2}.distances);
            results.error = [resPerFloor{1}.error; resPerFloor{2}.error];
            results.ncandidates = [resPerFloor{1}.ncandidates; resPerFloor{2}.ncandidates];
            results.prediction = [resPerFloor{1}.prediction; resPerFloor{2}.prediction];
            results.samples = [resPerFloor{1}.samples; resPerFloor{2}.samples];
            results.targets = [resPerFloor{1}.targets; resPerFloor{2}.targets];
            results.timefull = [resPerFloor{1}.timefull; resPerFloor{2}.timefull];
            results.timesample = [resPerFloor{1}.timesample; resPerFloor{2}.timesample];
        else
            results       = resPerFloor{1};
        end

		[area_test,area_estm] = setup_getArea(results,databasename);

		accuracy_accum  (i)   = mean(results.error(:,1));
		accuracies_accum(i,:) = results.error(:,1)';

		% Save results file
		%save([ dir_output, '/results_' sprintf('%03d',i) '.mat'],'database','results','area_test','area_estm','NperArea','-binary');

		area_test_accum{i} = area_test;
		area_estm_accum{i} = area_estm;

        NperArea_accum{i} = NperArea;

		clear database;
		clear simdatabase;

		distance_accum(i) = 0;
		for j = 1:testsamples
			distance_accum(i)     = distance_accum(i) + results.distances{j,1}(1);
			distances_accum(i,j) = results.distances{j,1}(1);
		end
		distance_accum(i) = distance_accum(i)/testsamples;
	end

	% Save resume file
    if IS_OCTAVE
	save([dir_output,  '/resume.mat'],...
	'accuracy_accum','distance_accum','accuracies_accum','distances_accum','area_test_accum','area_estm_accum','NperArea_accum','xEdges','yEdges','repetitions','testsamples','removedantenna','physicalaps','-binary');
	 else
	save([dir_output,  '/resume.mat'],...
	'accuracy_accum','distance_accum','accuracies_accum','distances_accum','area_test_accum','area_estm_accum','NperArea_accum','xEdges','yEdges','repetitions','testsamples','removedantenna','physicalaps');
    end
        
end
