% Copyright © 2017 Universitat Jaume I (UJI) and Tampere University of Technology (TUT)
%
% Permission is hereby granted, free of charge, to any person obtaining a copy of
% this software and associated documentation files (the "Software"), to deal in
% the Software without restriction, including without limitation the rights to
% use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
% of the Software, and to permit persons to whom the Software is furnished to do
% so, subject to the following conditions:
%
% The above copyright notice and this permission notice shall be included in all
% copies or substantial portions of the Software.
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
% OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
% SOFTWARE.

function run_remove_physicalantenna( databasename , removedantenna, IS_OCTAVE)

	more off; 
	
	% generate output folder 
	prefix_results = ['remove_physicalantenna' int2str(removedantenna)] ;
	dir_output     = ['results_'  databasename '_k_001' '/' prefix_results ];
	if ~exist(dir_output)
		mkdir(dir_output);
	end
    
 	% Set number of repetitions
	repetitions = setup_getRepetitions(databasename); 
  
	% load first subset to get some basic database features
	database    = loadDB(databasename, 1);
	testsamples = size(database.testMacs,1);	
	
	% generate output data structures
	accuracy_accum   = zeros(repetitions,1);
	distance_accum   = zeros(repetitions,1);
	accuracies_accum = zeros(repetitions,testsamples);
	distances_accum  = zeros(repetitions,testsamples);
  
	% modify macs according to macconfiguration and dataset 
	% this can be used to filter noisy APs or APs in other
	% buildings/floors/areas	
	physicalaps = setup_getPhysicalAP(databasename,database); 
		 
	for i = 1:repetitions
	  
		database0 = loadDB(databasename, i);

		database.trainingMacs   = database0.trainingMacs;
		database.trainingLabels = database0.trainingLabels;
		database.testMacs       = database0.testMacs;
		database.testLabels     = database0.testLabels;
		
		removedantennasmask = (physicalaps==removedantenna);
		database.testMacs(:,removedantennasmask) = +100;

		% Default k-NN with k=1, positive data representation
		% euclidean distance to compare fingerprints, and default
		% non-detected RSS value = 100
		results = knn_baseline(database,1,100,[]); 
		
		[area_test,area_estm] = setup_getArea(results,databasename); 

		accuracy_accum  (i)   = mean(results.error(:,1));
		accuracies_accum(i,:) = results.error(:,1)';

		% Save results file	
		%save([ dir_output '/results_' sprintf('%03d',i) '.mat'],'database','results','area_test','area_estm','-binary');

		area_test_accum{i} = area_test;
		area_estm_accum{i} = area_estm;

		clear database;
		clear simdatabase;

		distance_accum(i) = 0;
		for j = 1:testsamples
			distance_accum(i)     = distance_accum(i) + results.distances{j,1}(1);
			distances_accum(i,j) = results.distances{j,1}(1);
		end
		distance_accum(i) = distance_accum(i)/testsamples;			
	end
		
	% Save resume file		
	if IS_OCTAVE
		save([dir_output  '/resume.mat'],...
		'accuracy_accum','distance_accum','accuracies_accum','distances_accum','area_test_accum','area_estm_accum','repetitions','testsamples','removedantenna','physicalaps','-binary');
    else
		save([dir_output  '/resume.mat'],...
		'accuracy_accum','distance_accum','accuracies_accum','distances_accum','area_test_accum','area_estm_accum','repetitions','testsamples','removedantenna','physicalaps');
    end
end