% Copyright © 2017 Universitat Jaume I (UJI) and Tampere University of Technology (TUT)
%
% Permission is hereby granted, free of charge, to any person obtaining a copy of
% this software and associated documentation files (the "Software"), to deal in
% the Software without restriction, including without limitation the rights to
% use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
% of the Software, and to permit persons to whom the Software is furnished to do
% so, subject to the following conditions:
%
% The above copyright notice and this permission notice shall be included in all
% copies or substantial portions of the Software.
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
% OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
% SOFTWARE.



function [area_test,area_estm] = setup_getArea (results,database)

	% Default. No area assigned to estimates and test points
	area_test = 0*results.targets;
	area_estm = 0*results.targets;

	if strfind(database,'libdb')
		area_test = 1 + (results.targets(:,2)>=22.949)   + 2*((results.targets(:,3)-(3.7*5))<0.01);
		area_estm = 1 + (results.prediction(:,2)>22.949) + 2*((results.prediction(:,3)-(3.7*5))<0.01);
	elseif strfind(database,'simdb')
		area_test = 1 + (results.targets(:,1)>13.75) + (results.targets(:,1)>25.00) + (results.targets(:,1)>38.75) + 4*((results.targets(:,2)>10));
		area_estm = 1 + (results.prediction(:,1)>13.75) + (results.prediction(:,1)>25.00) + (results.prediction(:,1)>38.75) + 4*((results.prediction(:,2)>10));
	elseif strfind(database,'mandb')
		antennas = [
			-23.626,-18.596,1;
			8.538,-9.298,3;
			-10.702,-18.596,4;
			-1.93,-2.749,6;
			8.596,-14.62,8;
			7.135,6.023,9;
			21.17,-2.69,15;
			13.333,-2.69,16;
			32.398,-2.69,17;
			32.573,13.86,25
		];

		for i = 1:size(results.targets,1)
			distances = sum(((ones(10,1)*results.prediction(i,1:2)) - antennas(:,1:2)).^2,2);
			[~,area_estm(i)] = min(distances);
			distances = sum(((ones(10,1)*results.targets(i,1:2)) - antennas(:,1:2)).^2,2);
			[~,area_test(i)] = min(distances);
		end
	else
	  warning(sprintf('Area not found for database %s\n',database));
	end

end
